r"""Contain ``pandas.Series`` transformers."""

from __future__ import annotations

__all__ = [
    "BaseSeriesTransformer",
    "Sequential",
    "SequentialSeriesTransformer",
    "StripString",
    "StripStringSeriesTransformer",
    "ToDatetime",
    "ToDatetimeSeriesTransformer",
    "ToNumeric",
    "ToNumericSeriesTransformer",
    "is_series_transformer_config",
    "setup_series_transformer",
]

from flamme.transformer.series.base import (
    BaseSeriesTransformer,
    is_series_transformer_config,
    setup_series_transformer,
)
from flamme.transformer.series.datetime import ToDatetimeSeriesTransformer
from flamme.transformer.series.datetime import ToDatetimeSeriesTransformer as ToDatetime
from flamme.transformer.series.numeric import ToNumericSeriesTransformer
from flamme.transformer.series.numeric import ToNumericSeriesTransformer as ToNumeric
from flamme.transformer.series.sequential import SequentialSeriesTransformer
from flamme.transformer.series.sequential import (
    SequentialSeriesTransformer as Sequential,
)
from flamme.transformer.series.string import StripStringSeriesTransformer
from flamme.transformer.series.string import StripStringSeriesTransformer as StripString
